import 'dart:io';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/rendering.dart';

typedef NoArgsCallback = void Function();

enum MDResolution {
  Preset640x480,
  Preset960x540,
  Preset1080x720
}

enum MDOrientation {
  OrientationPortrait,
  OrientationLandscapeRight,
}

class MdpusherPlugin {
  static const MethodChannel _channel = const MethodChannel('mdpusher_plugin');
  bool isPortrait;
  bool frontCamera;
  bool flashLightOn;
  bool beautyOn;
  bool micOn;
  int frameRate;
  int bitrate;
  MDResolution resolution;

  NoArgsCallback onPreviewStarted;
  NoArgsCallback onPreviewStoped;
  NoArgsCallback onPushStarted;
  NoArgsCallback onPushStoped;
  NoArgsCallback onReconnectStart;
  NoArgsCallback onReconnectFail;
  NoArgsCallback onReconnectSucceed;
  NoArgsCallback onConnectionLost;
  NoArgsCallback onConnectFail;
  NoArgsCallback onSystemError;
  NoArgsCallback onSDKError;

  MdpusherPlugin({
    this.isPortrait,
    this.frontCamera,
    this.beautyOn,
    this.micOn,
    this.bitrate,
    this.frameRate,
    this.resolution
  }){
    if (isPortrait == null) {
      isPortrait = false;
    }
    if (frontCamera == null) {
      frontCamera = true;
    }
    if (beautyOn == null) {
      beautyOn = true;
    }
    if (micOn == null) {
      micOn = true;
    }
    if (bitrate == null) {
      bitrate = 500;
    }
    if (frameRate == null) {
      frameRate = 25;
    }
    if (resolution == null) {
      resolution = MDResolution.Preset960x540;
    }
    Map<String,String> param = new Map();
    param['portrait'] = this.isPortrait.toString();
    param['frontCamera'] = this.frontCamera.toString();
    param['beautyOn'] = this.beautyOn.toString();
    param['micOn'] = this.micOn.toString();
    param['frameRate'] = this.frameRate.toString();
    param['bitrate'] = this.bitrate.toString();
    param['resolution'] = this.resolution.index.toString();
    _channel.invokeMethod("config",param);
    _channel.setMethodCallHandler(methodCallHandler);
  }

  static Widget createNativeView(Function(int viewId) created) {
    if (Platform.isIOS) {
      return UiKitView(
        viewType: "mdview",
        onPlatformViewCreated: (int viewId){
          if (created != null) {
            created(viewId);
          }
        },
        hitTestBehavior: PlatformViewHitTestBehavior.transparent,
      );
    } else {
      return AndroidView(
        viewType: 'mdview',
        onPlatformViewCreated: (viewId) {
          if (created != null) {
            created(viewId);
          }
        },
        hitTestBehavior: PlatformViewHitTestBehavior.transparent,
      );
    }
  }

  void setCallback({
    NoArgsCallback onPreviewStarted,
    NoArgsCallback onPreviewStoped,
    NoArgsCallback onPushStarted,
    NoArgsCallback onPushStoped,
    NoArgsCallback onReconnectStart,
    NoArgsCallback onReconnectFail,
    NoArgsCallback onReconnectSucceed,
    NoArgsCallback onConnectionLost,
    NoArgsCallback onConnectFail,
    NoArgsCallback onSystemError,
    NoArgsCallback onSDKError,
  }) {
    this.onPreviewStarted = onPreviewStarted;
    this.onPreviewStoped = onPreviewStoped;
    this.onPushStarted = onPushStarted;
    this.onPushStoped = onPushStoped;
    this.onReconnectStart = onReconnectStart;
    this.onReconnectFail = onReconnectFail;
    this.onReconnectSucceed = onReconnectSucceed;
    this.onConnectionLost = onConnectionLost;
    this.onConnectFail = onConnectFail;
    this.onSystemError = onSystemError;
    this.onSDKError = onSDKError;
  }

  void startPreview(int viewId) async {
    assert(viewId != null);
    await _channel.invokeMethod('startPreview',{'viewId':viewId.toString()});
  }

  void stopPreview() async {
    await _channel.invokeMethod('stopPreview');
  }

  void startPush(String url) async {
    assert(url != null);
    await _channel.invokeMethod('startPush',{'url':url});
  }

  void startPushAsync(String url) async {
    assert(url != null);
    await _channel.invokeMethod('startPushAsync',{'url':url});
  }

  void stopPush() async {
    await _channel.invokeMethod('stopPush');
  }

  void switchCamera() async {
    await _channel.invokeMethod('switchCamera');
  }

  void setFlashLight(bool flashLightOn) async {
    await _channel.invokeMethod('setFlashLight',{'flashLight':flashLightOn.toString()});
  }

  void setBeautyOn(bool beautyOn) async {
    await _channel.invokeMethod('setBeautyOn',{'beautyOn':beautyOn.toString()});
  }

  void setMicOn(bool micOn) async {
    await _channel.invokeMethod('setMicOn',{'micOn':micOn.toString()});
  }
  
  static void setOrientation(MDOrientation orientation) async {
    if (Platform.isIOS) {
      if(orientation != null) {
        await _channel.invokeMethod('changeOrientation',{'orientation':orientation.index.toString()});
      } else {
        await _channel.invokeMethod('changeOrientation');
      }
    } else {
      if (orientation != null) {
        if (orientation == MDOrientation.OrientationPortrait) {
          await SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp]);
        } else {
          await SystemChrome.setPreferredOrientations([DeviceOrientation.landscapeRight]);
        }
      } else {
        await SystemChrome.setPreferredOrientations([]);
      }
    }
  }

  void destory() async {
    await _channel.invokeMethod('destory');
    _channel.setMethodCallHandler(null);
  }

  Future<void> methodCallHandler(MethodCall call) {
    switch (call.method) {
      case 'onPreviewStarted':
        if (onPreviewStarted != null) {
          onPreviewStarted();
        }
        break;
      case 'onPreviewStoped':
        if (onPreviewStoped != null) {
          onPreviewStoped();
        }
        break;
      case 'onPushStarted':
        if (onPushStarted != null) {
          onPushStarted();
        }
        break;
      case 'onPushStoped':
        if (onPushStoped != null) {
          onPushStoped();
        }
        break;
      case 'onReconnectStart':
        if (onReconnectStart != null) {
          onReconnectStart();
        }
        break;
      case 'onReconnectFail':
        if (onReconnectFail != null) {
          onReconnectFail();
        }
        break;
      case 'onReconnectSucceed':
        if (onReconnectSucceed != null) {
          onReconnectSucceed();
        }
        break;
      case 'onConnectionLost':
        if (onConnectionLost != null) {
          onConnectionLost();
        }
        break;
      case 'onConnectFail':
        if (onConnectFail != null) {
          onConnectFail();
        }
        break;
      case 'onSystemError':
        if (onSystemError != null) {
          onSystemError();
        }
        break;
      case 'onSDKError':
        if (onSDKError != null) {
          onSDKError();
        }
        break;
    }
  }

}